@@ -1,20 +1,17 @@
 # coding: utf-8
 
-# PYTHON IMPORTS
-import re
 import os
-import unicodedata
+import re
 
-# DJANGO IMPORTS
 from django import forms
 from django.utils.translation import ugettext_lazy as _
 
-# FILEBROWSER IMPORTS
-from filebrowser.settings import FOLDER_REGEX, NORMALIZE_FILENAME, CONVERT_FILENAME
+from filebrowser.settings import FOLDER_REGEX
+from filebrowser.utils import convert_filename
+
 
 ALNUM_NAME_RE = re.compile(FOLDER_REGEX, re.U)
 
-# CHOICES
 TRANSPOSE_CHOICES = (
     ("", u"-----"),
     ("0", _(u"Flip horizontal")),
@@ -25,30 +22,6 @@ TRANSPOSE_CHOICES = (
 )
 
 
-def convert_filename(value):
-    """
-    Convert Filename.
-    """
-
-    if NORMALIZE_FILENAME:
-        chunks = value.split(os.extsep)
-        normalized = []
-        for v in chunks:
-            v = unicodedata.normalize('NFKD', unicode(v)).encode('ascii', 'ignore')
-            v = re.sub(r'[^\w\s-]', '', v).strip()
-            normalized.append(v)
-
-        if len(normalized) > 1:
-            value = '.'.join(normalized)
-        else:
-            value = normalized[0]
-
-    if CONVERT_FILENAME:
-        value = value.replace(" ", "_").lower()
-
-    return value
-
-
 class CreateDirForm(forms.Form):
     """
     Form for creating a folder.
